package bug.app.dispose;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IProduct;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.application.WorkbenchAdvisor;
import org.eclipse.ui.branding.IProductConstants;
import org.osgi.framework.Bundle;

/**
 * This class controls all aspects of the application's execution and is
 * contributed through the plugin.xml.
 */
public class Application implements IApplication {

	public Object start(IApplicationContext context) throws Exception {
		Display display = null;
		Image[] images = null;
		try {
			display = PlatformUI.createDisplay();
			images = setWindowDefaultImages();
			WorkbenchAdvisor advisor = new ApplicationWorkbenchAdvisor();
			int returnCode = PlatformUI.createAndRunWorkbench(display, advisor);
			if (returnCode == PlatformUI.RETURN_RESTART) {
				return IApplication.EXIT_RESTART;
			}
			return IApplication.EXIT_OK;
		}
		catch (Exception e) {
			return getApplicationStatus(e);
		}
		finally {
			dispose(display, images);
		}
	}

	private int getApplicationStatus(Exception e) throws Exception {
		Throwable cause = e.getCause();
		e.printStackTrace();
		if (cause != null && cause.getMessage() != null && cause.getMessage().contains("401")) {
			return IApplication.EXIT_OK;
		}
		if (cause instanceof Error) {
			throw (Error) e.getCause();
		}
		else {
			throw e;
		}
	}

	private void dispose(Display display, Image[] images) {
		if (images != null) {
			for (Image image : images) {
				try {
					if (image != null && !image.isDisposed()) {
						image.dispose();
					}
				}
				catch (Exception e) {
					// ignore
				}
			}
		}
		try {
			if (display != null && !display.isDisposed()) {
				display.dispose();
			}
		}
		catch (Exception e) {
			// ignore
		}
	}

	private Image[] setWindowDefaultImages() {
		String[] imagesPaths = new String[]{};
		IProduct product = Platform.getProduct();
		Bundle definingBundle = null;
		// If a product is available, read it. This is always the case for RCP.
		if (product != null) {
			definingBundle = product.getDefiningBundle();
			String windowImages = product.getProperty(IProductConstants.WINDOW_IMAGES);
			imagesPaths = windowImages.split(",");
		}
		// If no product is available, use the extension mechanism to find the
		// product definition.
		// This is always the case for RAP.
		else {
			IExtensionPoint extensionPoint = Platform.getExtensionRegistry().getExtensionPoint("org.eclipse.core.runtime.products");
			if (extensionPoint != null) {
				IExtension[] extensions = extensionPoint.getExtensions();
				if (extensions.length == 1) {
					IExtension extension = extensions[0];
					IConfigurationElement[] configurationElements = extension.getConfigurationElements();
					if (configurationElements.length == 1) {
						IConfigurationElement configurationElement = configurationElements[0];
						IConfigurationElement[] childConfigurationElements = configurationElement.getChildren();
						for (IConfigurationElement childConfigurationElement : childConfigurationElements) {
							if (childConfigurationElement.getAttribute("name").equalsIgnoreCase(IProductConstants.WINDOW_IMAGES)) {
								definingBundle = Platform.getBundle(extension.getContributor().getName());
								String windowImages = childConfigurationElement.getAttribute("value");
								imagesPaths = windowImages.split(",");
								break;
							}
						}
					}
				}
			}
		}

		Image[] images = createImages(imagesPaths, definingBundle);

		// emergency exit if one of the icons is null.
		for (Image image : images) {
			if (image == null) {
				return images;
			}
		}

		Window.setDefaultImages(images);

		return images;
	}

	private Image[] createImages(String[] imagesPaths, Bundle definingBundle) {
		List<Image> images = new ArrayList<Image>();

		for (int i = 0; i < imagesPaths.length; i++) {
			ImageDescriptor imageDescriptor = createImageDescriptor(imagesPaths[i], definingBundle);
			if (imageDescriptor != null) {
				images.add(imageDescriptor.createImage());
			}
		}

		return images.toArray(new Image[images.size()]);
	}

	public final ImageDescriptor createImageDescriptor(String iconPath, Bundle bundle) {
		URL customEntry = FileLocator.find(bundle, new Path(iconPath), new HashMap());
		if (customEntry != null) {
			return ImageDescriptor.createFromURL(customEntry);
		}
		return null;
	}

	public void stop() {
		if (!PlatformUI.isWorkbenchRunning()) {
			return;
		}
		final IWorkbench workbench = PlatformUI.getWorkbench();
		final Display display = workbench.getDisplay();
		display.syncExec(new Runnable() {

			@Override
			public void run() {
				if (!display.isDisposed()) {
					workbench.close();
				}
			}
		});
	}
}
